// ========			messiahCommand.cpp			==============
//
// Copyright (C) 2002 pmG Worldwide LLC. All Rights Reserved.
//
// ===============================================================
//
// Description: 
//
// Implementation of messiahCommand.
//
// ===============================================================
//
// Version History:
//
// Written By: Ron Griswold
// Date: 4/18/2002
//
// Revised By:
// Date: 
//
// ===============================================================
#include "messiahCommand.h"

#include <maya/MArgList.h>

#include <maya/MGlobal.h>
#include <maya/MDagPath.h>
#include <maya/MItSelectionList.h>
#include <maya/MSelectionList.h>

#include <maya/MFnDependencyNode.h>
#include <maya/MDGModifier.h>

int	temporary_messiahobj_global = 0;

// ===============================================================
// Constructor
// ===============================================================
messiahCommand::messiahCommand() : _argFlags(0)
{

}

// ===============================================================
// Destructor
// ===============================================================
messiahCommand::~messiahCommand()
{

}

// ===============================================================
// Creator
// ===============================================================
void *	messiahCommand::creator()
{
	return new messiahCommand();
}

// ===============================================================
// Parse Args
// ===============================================================
MStatus		messiahCommand::parseArgs(MArgList const &args)
{
	MStatus     	stat;
	MString     	arg;


	for( int i = 0; i < args.length(); i++)
	{
		arg = args.asString( i, &stat );
		if(!stat)
			continue;

		if( arg == MM_ARG_HELP_S || arg == MM_ARG_HELP_L )
		{
			this->_argFlags |= MC_HELP;
		}
		if( arg == MM_ARG_ATTACH_XFORM_S || arg == MM_ARG_ATTACH_XFORM_L )
		{
			this->_argFlags |= MC_ATTACH_XFORM;
		}
		if( arg == MM_ARG_ATTACH_DFORM_S || arg == MM_ARG_ATTACH_DFORM_L )
		{
			this->_argFlags |= MC_ATTACH_DFORM;
		}
		if( arg == MM_ARG_QUERY_S || arg == MM_ARG_QUERY_L )
		{
			this->_argFlags |= MC_QUERY;
		}
		if( arg == MM_ARG_SET_SCALE_S || arg == MM_ARG_SET_SCALE_L )
		{
			if( !(this->_argFlags & MC_QUERY) )
			{
				double factor = args.asDouble(++i, &stat);
				if(stat) MHS::scaleFactor = factor;
			}
			this->_argFlags |= MC_SET_SCALE;
		}


	}// foreach arg

	return MS::kSuccess;
}

// ===============================================================
// Do It
// ===============================================================
MStatus		messiahCommand::doIt(MArgList const &args)
{
	MStatus stat;

	stat = this->parseArgs(args);
	if( !stat )
		return stat;

	this->redoIt();

	return MS::kSuccess;
}

// ===============================================================
// Redo It
// ===============================================================
MStatus		messiahCommand::redoIt()
{
	MStatus	stat;

	// display messiah's interface ///////////////////
	if( this->_argFlags == 0 )
	{
		MHS *messiah = NULL;
		if( !(messiah = MHS::Inst()) )
		{
			MGlobal::displayError( MString("messiah not loaded. Use 'messiah -startup' to load messiah"));
			return MS::kFailure;
		}

		if( !(messiah->ShowInterface()) )
		{
			MGlobal::displayError( MString("Unable to launch messiah interface.") );
			return MS::kFailure;
		}
		
		return MS::kSuccess;
	}
	/////////////////////////////////////////////////

	// Return Help Info /////////////////////////////
	if( this->_argFlags & MC_HELP )
	{
		this->clearResult();
		
		this->appendToResult("messiahMaya Command Usage:\n");
		this->appendToResult("\tDisplay Interface = no arguments\n");
		this->appendToResult("\tAttach Deformer to Selected = -attachDform or -ad\n");
		this->appendToResult("\tAttach Transformer to Selected = -attachXform or -ax\n");
		this->appendToResult("\tSet Scale Factor = -scaleFactor [value] or -sf [value]\n");
		this->appendToResult("\tQuery Scale Factor = -q -scaleFactor or -q -sf\n");

		return MS::kSuccess;
	}
	/////////////////////////////////////////////////

	// Return Scale Factor //////////////////////////
	if( this->_argFlags & MC_QUERY && this->_argFlags & MC_SET_SCALE )
	{
		this->clearResult();
		this->setResult(MHS::scaleFactor);
		
		return MS::kSuccess;
	}
	/////////////////////////////////////////////////

	// Ensure that -ax and -ad flags are not both set ////////////////////////
	if( (this->_argFlags & messiahCommand::MC_ATTACH_XFORM) && (this->_argFlags & messiahCommand::MC_ATTACH_DFORM) )
	{
		MGlobal::displayError(MString( "-ax and -ad flags are mutually exclusive" ));
		return MS::kFailure;
	}
	//////////////////////////////////////////////////////////////////////////

	// Attach Dform Node /////////////////////////////////////////////////////
	if( this->_argFlags & MC_ATTACH_DFORM )
	{
		MFnDependencyNode	dNode;

		MSelectionList	list;
		MGlobal::getActiveSelectionList( list );
		MItSelectionList iter( list );

		MString	createCommand("deformer -type messiahDform -n ");
		MString nodeName;

		MObject		mayaItem;

		while( !iter.isDone() )
		{
			iter.getDependNode( mayaItem );

			dNode.setObject( mayaItem );

			int id = dNode.type();

			if( dNode.type() )//== MFn::kTransform )
			{

				nodeName = dNode.name();
				nodeName += "_messiahDform";

				createCommand += nodeName;

				// execute deform command, pass nodeName in case Maya renames the node
				MGlobal::executeCommand(createCommand, nodeName);

				//
				// connect hostItem.message to messiahDform.hostItem
				//
				MDGModifier	dgMod;
				MObject		outPlug;
				MObject		inPlug;

				outPlug = dNode.attribute(MString("message"), &stat);
				if(!stat)
					MGlobal::displayError("failed to select message plug");

				// select the newly created deform node
				MSelectionList	selTmp;
				stat = selTmp.add( nodeName );
				if(!stat)
					MGlobal::displayError(MString("failed to select deformer node"));
				MItSelectionList selItTmp( selTmp );

				MObject dformer;
				stat = selItTmp.getDependNode( dformer );
				if(!stat)
					MGlobal::displayError(MString("failed to select deformer MObject"));

				dNode.setObject( dformer );
				inPlug = dNode.attribute(MString("hostItem"), &stat);
				if(!stat)
					MGlobal::displayError("failed to select hostItem plug");

				dgMod.connect( mayaItem, outPlug, dformer, inPlug );
				stat = dgMod.doIt();
				if(!stat)
					MGlobal::displayError(MString("failed to connect hostItem"));

				// connect time1.outTime to dformer.time
				MSelectionList		tmpSelList;
				tmpSelList.add(MString("time1"));
				MItSelectionList	tmpIter( tmpSelList );

				MObject time1;
				tmpIter.getDependNode( time1 );

				dNode.setObject( time1 );
				outPlug = dNode.attribute(MString("outTime"));
				dNode.setObject( dformer );
				inPlug = dNode.attribute(MString("time"));
				dgMod.connect( time1, outPlug, dformer, inPlug);
				dgMod.doIt();
				///////////////////////////////////////////////////////////
			}// if transform node
			else
				MGlobal::displayError("Messiah nodes can only be attached to a transform node!");

			iter.next();
		}
	}

	// Attach Xform Node ///////////////////////////////////////////////
	if( this->_argFlags & MC_ATTACH_XFORM )
	{
		messiahCommand *cmd = this;

		MFnDependencyNode	dNode;
		
		MSelectionList	list;
		MGlobal::getActiveSelectionList( list );
		MItSelectionList iter( list );

		MObject		mayaItem;

		while( !iter.isDone() )
		{
			iter.getDependNode( mayaItem );

			// create messiahXform node
			MObject xformer = dNode.create( MM_MOVE_ID );

			// set the xform node's name
			MString nodeName;
			dNode.setObject( mayaItem );
			nodeName = dNode.name();
			nodeName += "_messiahXform";
			
			dNode.setObject( xformer );
			dNode.setName( nodeName );

			// connect the attributes
			MDGModifier	dgMod;
			MObject		outPlug;
			MObject		inPlug;

			// connect rotation//////////////////////////////////////
			dNode.setObject( xformer );
			outPlug = dNode.attribute(MString("rotateX"), &stat);
			dNode.setObject( mayaItem );
			inPlug  = dNode.attribute(MString("rotateX"), &stat);
			stat = dgMod.connect( xformer, outPlug, mayaItem, inPlug );

			dNode.setObject( xformer );
			outPlug = dNode.attribute(MString("rotateY"), &stat);
			dNode.setObject( mayaItem );
			inPlug  = dNode.attribute(MString("rotateY"), &stat);
			stat = dgMod.connect( xformer, outPlug, mayaItem, inPlug );

			dNode.setObject( xformer );
			outPlug = dNode.attribute(MString("rotateZ"), &stat);
			dNode.setObject( mayaItem );
			inPlug  = dNode.attribute(MString("rotateZ"), &stat);
			stat = dgMod.connect( xformer, outPlug, mayaItem, inPlug );

			dNode.setObject( xformer );
			outPlug = dNode.attribute(MString("rotateOrder"), &stat);
			dNode.setObject( mayaItem );
			inPlug  = dNode.attribute(MString("rotateOrder"), &stat);
			stat = dgMod.connect( xformer, outPlug, mayaItem, inPlug );

			stat = dgMod.doIt();
			///////////////////////////////////////////////////////////

			// connect transformation /////////////////////////////////
			dNode.setObject( xformer );
			outPlug = dNode.attribute(MString("translateX"), &stat);
			dNode.setObject( mayaItem );
			inPlug  = dNode.attribute(MString("translateX"), &stat);
			stat = dgMod.connect( xformer, outPlug, mayaItem, inPlug );

			dNode.setObject( xformer );
			outPlug = dNode.attribute(MString("translateY"), &stat);
			dNode.setObject( mayaItem );
			inPlug  = dNode.attribute(MString("translateY"), &stat);
			stat = dgMod.connect( xformer, outPlug, mayaItem, inPlug );

			dNode.setObject( xformer );
			outPlug = dNode.attribute(MString("translateZ"), &stat);
			dNode.setObject( mayaItem );
			inPlug  = dNode.attribute(MString("translateZ"), &stat);
			stat = dgMod.connect( xformer, outPlug, mayaItem, inPlug );

			stat = dgMod.doIt();
			///////////////////////////////////////////////////////////

			// connect host item message //////////////////////////////
			dNode.setObject( mayaItem );
			outPlug = dNode.attribute(MString("message"), &stat);
			dNode.setObject( xformer );
			inPlug = dNode.attribute(MString("hostItem"), &stat);
			stat = dgMod.connect( mayaItem, outPlug, xformer, inPlug);
			stat = dgMod.doIt();
			///////////////////////////////////////////////////////////

			// connect time1.outTime to xformer.time /////////////////
			MSelectionList		tmpSelList;
			tmpSelList.add(MString("time1"));
			MItSelectionList	tmpIter( tmpSelList );

			MObject time1;
			tmpIter.getDependNode( time1 );

			dNode.setObject( time1 );
			outPlug = dNode.attribute(MString("outTime"));
			dNode.setObject( xformer );
			inPlug = dNode.attribute(MString("time"));
			dgMod.connect( time1, outPlug, xformer, inPlug);
			dgMod.doIt();
			///////////////////////////////////////////////////////////

			iter.next();
		}

	}
	//////////////////////////////////////////////////
	return MS::kSuccess;
}

// ===============================================================
// Undo It
// ===============================================================
MStatus		messiahCommand::undoIt()
{

	//TODO: Implement Undo
	return MS::kSuccess;
}
